@@ -430,8 +430,11 @@ func (n Comment) String() string {
 
 // JS writes JavaScript to writer.
 func (n Comment) JS(w io.Writer) {
-	w.Write(n.Value)
-	w.Write([]byte("\n"))
+	if wi, ok := w.(Indenter); ok {
+		wi.w.Write(n.Value)
+	} else {
+		w.Write(n.Value)
+	}
 }
 
 // BlockStmt is a block statement.
@@ -555,9 +558,11 @@ func (n DoWhileStmt) JS(w io.Writer) {
 	}
 	n.Body.JS(w)
 	if _, ok := n.Body.(*VarDecl); ok {
-		w.Write([]byte(";"))
+		w.Write([]byte("; "))
+	} else if _, ok := n.Body.(*Comment); !ok {
+		w.Write([]byte(" "))
 	}
-	w.Write([]byte(" while ("))
+	w.Write([]byte("while ("))
 	n.Cond.JS(w)
 	w.Write([]byte(");"))
 }
@@ -577,9 +582,11 @@ func (n WhileStmt) JS(w io.Writer) {
 	w.Write([]byte("while ("))
 	n.Cond.JS(w)
 	w.Write([]byte(")"))
-	if _, ok := n.Body.(*EmptyStmt); !ok {
-		w.Write([]byte(" "))
+	if _, ok := n.Body.(*EmptyStmt); ok {
+		w.Write([]byte(";"))
+		return
 	}
+	w.Write([]byte(" "))
 	n.Body.JS(w)
 	if _, ok := n.Body.(*VarDecl); ok {
 		w.Write([]byte(";"))
@@ -974,6 +981,9 @@ func (n ImportStmt) String() string {
 
 // JS writes JavaScript to writer.
 func (n ImportStmt) JS(w io.Writer) {
+	if wi, ok := w.(Indenter); ok {
+		w = wi.w
+	}
 	w.Write([]byte("import"))
 	if n.Default != nil {
 		w.Write([]byte(" "))
@@ -1047,6 +1057,9 @@ func (n ExportStmt) String() string {
 
 // JS writes JavaScript to writer.
 func (n ExportStmt) JS(w io.Writer) {
+	if wi, ok := w.(Indenter); ok {
+		w = wi.w
+	}
 	w.Write([]byte("export"))
 	if n.Decl != nil {
 		if n.Default {
@@ -1092,6 +1105,9 @@ func (n DirectivePrologueStmt) String() string {
 
 // JS writes JavaScript to writer.
 func (n DirectivePrologueStmt) JS(w io.Writer) {
+	if wi, ok := w.(Indenter); ok {
+		w = wi.w
+	}
 	w.Write(n.Value)
 	w.Write([]byte(";"))
 }
@@ -1160,6 +1176,9 @@ func (n PropertyName) JS(w io.Writer) {
 		w.Write([]byte("]"))
 		return
 	}
+	if wi, ok := w.(Indenter); ok {
+		w = wi.w
+	}
 	w.Write(n.Literal.Data)
 }
 
@@ -1182,6 +1201,8 @@ func (n BindingArray) String() string {
 			s += ","
 		}
 		s += " ...Binding(" + n.Rest.String() + ")"
+	} else if 0 < len(n.List) && n.List[len(n.List)-1].Binding == nil {
+		s += ","
 	}
 	return s + " ]"
 }
@@ -1191,9 +1212,14 @@ func (n BindingArray) JS(w io.Writer) {
 	w.Write([]byte("["))
 	for j, item := range n.List {
 		if j != 0 {
-			w.Write([]byte(", "))
+			w.Write([]byte(","))
+		}
+		if item.Binding != nil {
+			if j != 0 {
+				w.Write([]byte(" "))
+			}
+			item.JS(w)
 		}
-		item.JS(w)
 	}
 	if n.Rest != nil {
 		if len(n.List) != 0 {
@@ -1201,6 +1227,8 @@ func (n BindingArray) JS(w io.Writer) {
 		}
 		w.Write([]byte("..."))
 		n.Rest.JS(w)
+	} else if 0 < len(n.List) && n.List[len(n.List)-1].Binding == nil {
+		w.Write([]byte(","))
 	}
 	w.Write([]byte("]"))
 }
@@ -1632,12 +1660,15 @@ func (n LiteralExpr) String() string {
 
 // JS writes JavaScript to writer.
 func (n LiteralExpr) JS(w io.Writer) {
+	if wi, ok := w.(Indenter); ok {
+		w = wi.w
+	}
 	w.Write(n.Data)
 }
 
 // JSON writes JSON to writer.
 func (n LiteralExpr) JSON(w io.Writer) error {
-	if n.TokenType == TrueToken || n.TokenType == FalseToken || n.TokenType == NullToken || n.TokenType == DecimalToken {
+	if n.TokenType == TrueToken || n.TokenType == FalseToken || n.TokenType == NullToken || n.TokenType == DecimalToken || n.TokenType == IntegerToken {
 		w.Write(n.Data)
 		return nil
 	} else if n.TokenType == StringToken {
@@ -1894,6 +1925,9 @@ func (n TemplateExpr) String() string {
 
 // JS writes JavaScript to writer.
 func (n TemplateExpr) JS(w io.Writer) {
+	if wi, ok := w.(Indenter); ok {
+		w = wi.w
+	}
 	if n.Tag != nil {
 		n.Tag.JS(w)
 		if n.Optional {
@@ -1967,7 +2001,7 @@ func (n DotExpr) String() string {
 // JS writes JavaScript to writer.
 func (n DotExpr) JS(w io.Writer) {
 	lit, ok := n.X.(*LiteralExpr)
-	group := ok && !n.Optional && lit.TokenType == DecimalToken
+	group := ok && !n.Optional && (lit.TokenType == DecimalToken || lit.TokenType == IntegerToken)
 	if group {
 		w.Write([]byte("("))
 	}
@@ -2127,7 +2161,7 @@ func (n UnaryExpr) JS(w io.Writer) {
 		n.X.JS(w)
 		w.Write(n.Op.Bytes())
 		return
-	} else if unary, ok := n.X.(*UnaryExpr); ok && (n.Op == PosToken && unary.Op == PreIncrToken || n.Op == NegToken && unary.Op == PreDecrToken) || IsIdentifierName(n.Op) {
+	} else if unary, ok := n.X.(*UnaryExpr); ok && (n.Op == PosToken && (unary.Op == PreIncrToken || unary.Op == PosToken) || n.Op == NegToken && (unary.Op == PreDecrToken || unary.Op == NegToken)) || IsIdentifierName(n.Op) {
 		w.Write(n.Op.Bytes())
 		w.Write([]byte(" "))
 		n.X.JS(w)
@@ -2139,7 +2173,7 @@ func (n UnaryExpr) JS(w io.Writer) {
 
 // JSON writes JSON to writer.
 func (n UnaryExpr) JSON(w io.Writer) error {
-	if lit, ok := n.X.(*LiteralExpr); ok && n.Op == NegToken && lit.TokenType == DecimalToken {
+	if lit, ok := n.X.(*LiteralExpr); ok && n.Op == NegToken && (lit.TokenType == DecimalToken || lit.TokenType == IntegerToken) {
 		w.Write([]byte("-"))
 		w.Write(lit.Data)
 		return nil
