@@ -119,7 +119,12 @@ func TestJS(t *testing.T) {
 		{"{}\n;", "{} ;"},
 		{"- - --3", "- - --3;"},
 		{"([,,])=>P", "([,,]) => { return P; };"},
-		{"(t)=>{//!\n}", "(t) => { //! };"},
+		{"(t)=>{//!\n}", "(t) => { //! };"}, // space after //! is newline
+		{"import();", "import();"},
+		{"0\n.k", "(0).k;"},
+		{"do//!\n; while(1)", "//! do; while (1);"},           // space after //! is newline
+		{"//!\nn=>{ return n }", "//! (n) => { return n; };"}, // space after //! is newline
+		{"//!\n{//!\n}", "//! { //! }"},                       // space after //! is newline
 	}
 
 	re := regexp.MustCompile("\n *")
