@@ -27,6 +27,7 @@ type Parser struct {
 	in, await, yield, deflt, retrn bool
 	assumeArrowFunc                bool
 	allowDirectivePrologue         bool
+	comments                       []IStmt
 
 	stmtLevel int
 	exprLevel int
@@ -74,16 +75,13 @@ func Parse(r *parse.Input, o Options) (*AST, error) {
 		}
 	}
 
-	if p.err == nil {
-		p.err = p.l.Err()
-	} else {
+	if p.err != nil {
 		offset := p.l.r.Offset() - len(p.data)
-		p.err = parse.NewError(buffer.NewReader(p.l.r.Bytes()), offset, p.err.Error())
+		return nil, parse.NewError(buffer.NewReader(p.l.r.Bytes()), offset, p.err.Error())
+	} else if p.l.Err() != nil && p.l.Err() != io.EOF {
+		return nil, p.l.Err()
 	}
-	if p.err == io.EOF {
-		p.err = nil
-	}
-	return ast, p.err
+	return ast, nil
 }
 
 ////////////////////////////////////////////////////////////////
@@ -91,9 +89,22 @@ func Parse(r *parse.Input, o Options) (*AST, error) {
 func (p *Parser) next() {
 	p.prevLT = false
 	p.tt, p.data = p.l.Next()
-	for p.tt == WhitespaceToken || p.tt == LineTerminatorToken || (p.tt == CommentToken || p.tt == CommentLineTerminatorToken) && (p.exprLevel != 0 || len(p.data) < 3 || p.data[2] != '!') {
-		if p.tt == LineTerminatorToken || p.tt == CommentLineTerminatorToken {
+Loop:
+	for {
+		switch p.tt {
+		case WhitespaceToken:
+			// no-op
+		case LineTerminatorToken:
 			p.prevLT = true
+		case CommentToken, CommentLineTerminatorToken:
+			if 2 < len(p.data) && p.data[2] == '!' {
+				p.comments = append(p.comments, &Comment{p.data})
+			}
+			if p.tt == CommentLineTerminatorToken {
+				p.prevLT = true
+			}
+		default:
+			break Loop
 		}
 		p.tt, p.data = p.l.Next()
 	}
@@ -180,6 +191,10 @@ func (p *Parser) parseModule() (module BlockStmt) {
 	for {
 		switch p.tt {
 		case ErrorToken:
+			if 0 < len(p.comments) {
+				module.List = append(p.comments, module.List...)
+				p.comments = p.comments[:0]
+			}
 			return
 		case ImportToken:
 			p.next()
@@ -190,6 +205,9 @@ func (p *Parser) parseModule() (module BlockStmt) {
 				suffix := p.parseExpressionSuffix(left, OpExpr, OpCall)
 				p.exprLevel--
 				module.List = append(module.List, &ExprStmt{suffix})
+				if !p.prevLT && p.tt == SemicolonToken {
+					p.next()
+				}
 			} else {
 				importStmt := p.parseImportStmt()
 				module.List = append(module.List, &importStmt)
@@ -355,12 +373,55 @@ func (p *Parser) parseStmt(allowDeclaration bool) (stmt IStmt) {
 				}
 			}
 			init = varDecl
+		} else if await {
+			init = p.parseExpression(OpLHS)
 		} else if p.tt != SemicolonToken {
 			init = p.parseExpression(OpExpr)
 		}
 		p.in = true
 
-		if p.tt == SemicolonToken {
+		isLHSExpr := isLHSExpr(init)
+		if isLHSExpr && p.tt == InToken {
+			if await {
+				p.fail("for statement", OfToken)
+				return
+			}
+			p.next()
+			value := p.parseExpression(OpExpr)
+			if !p.consume("for statement", CloseParenToken) {
+				return
+			}
+			p.scope.MarkForStmt()
+			if p.tt == OpenBraceToken {
+				body.List = p.parseStmtList("")
+			} else if p.tt != SemicolonToken {
+				body.List = []IStmt{p.parseStmt(false)}
+			} else {
+				p.next()
+			}
+			if varDecl, ok := init.(*VarDecl); ok {
+				varDecl.InForInOf = true
+			}
+			stmt = &ForInStmt{init, value, body}
+		} else if isLHSExpr && p.tt == OfToken {
+			p.next()
+			value := p.parseExpression(OpAssign)
+			if !p.consume("for statement", CloseParenToken) {
+				return
+			}
+			p.scope.MarkForStmt()
+			if p.tt == OpenBraceToken {
+				body.List = p.parseStmtList("")
+			} else if p.tt != SemicolonToken {
+				body.List = []IStmt{p.parseStmt(false)}
+			} else {
+				p.next()
+			}
+			if varDecl, ok := init.(*VarDecl); ok {
+				varDecl.InForInOf = true
+			}
+			stmt = &ForOfStmt{await, init, value, body}
+		} else if p.tt == SemicolonToken {
 			var cond, post IExpr
 			if await {
 				p.fail("for statement", OfToken)
@@ -395,49 +456,12 @@ func (p *Parser) parseStmt(allowDeclaration bool) (stmt IStmt) {
 				varDecl.InFor = true
 			}
 			stmt = &ForStmt{init, cond, post, body}
-		} else if p.tt == InToken {
-			if await {
-				p.fail("for statement", OfToken)
-				return
-			}
-			p.next()
-			value := p.parseExpression(OpExpr)
-			if !p.consume("for statement", CloseParenToken) {
-				return
-			}
-			p.scope.MarkForStmt()
-			if p.tt == OpenBraceToken {
-				body.List = p.parseStmtList("")
-			} else if p.tt != SemicolonToken {
-				body.List = []IStmt{p.parseStmt(false)}
-			} else {
-				p.next()
-			}
-			if varDecl, ok := init.(*VarDecl); ok {
-				varDecl.InForInOf = true
-			}
-			stmt = &ForInStmt{init, value, body}
-		} else if p.tt == OfToken {
-			p.next()
-			value := p.parseExpression(OpAssign)
-			if !p.consume("for statement", CloseParenToken) {
-				return
-			}
-			p.scope.MarkForStmt()
-			if p.tt == OpenBraceToken {
-				body.List = p.parseStmtList("")
-			} else if p.tt != SemicolonToken {
-				body.List = []IStmt{p.parseStmt(false)}
-			} else {
-				p.next()
-			}
-			if varDecl, ok := init.(*VarDecl); ok {
-				varDecl.InForInOf = true
-			}
-			stmt = &ForOfStmt{await, init, value, body}
-		} else {
+		} else if isLHSExpr {
 			p.fail("for statement", InToken, OfToken, SemicolonToken)
 			return
+		} else {
+			p.fail("for statement", SemicolonToken)
+			return
 		}
 		p.exitScope(parent)
 	case SwitchToken:
@@ -558,10 +582,6 @@ func (p *Parser) parseStmt(allowDeclaration bool) (stmt IStmt) {
 	case SemicolonToken:
 		stmt = &EmptyStmt{}
 		p.next()
-	case CommentToken, CommentLineTerminatorToken:
-		// bang comment
-		stmt = &Comment{p.data}
-		p.next()
 	case ErrorToken:
 		stmt = &EmptyStmt{}
 		return
@@ -613,6 +633,7 @@ func (p *Parser) parseStmt(allowDeclaration bool) (stmt IStmt) {
 }
 
 func (p *Parser) parseStmtList(in string) (list []IStmt) {
+	comments := len(p.comments)
 	if !p.consume(in, OpenBraceToken) {
 		return
 	}
@@ -626,6 +647,13 @@ func (p *Parser) parseStmtList(in string) (list []IStmt) {
 		}
 		list = append(list, p.parseStmt(true))
 	}
+	if comments < len(p.comments) {
+		list2 := make([]IStmt, 0, len(p.comments)-comments+len(list))
+		list2 = append(list2, p.comments[comments:]...)
+		list2 = append(list2, list...)
+		list = list2
+		p.comments = p.comments[:comments]
+	}
 	return
 }
 
@@ -800,15 +828,17 @@ func (p *Parser) parseExportStmt() (exportStmt ExportStmt) {
 		exportStmt.Default = true
 		p.next()
 		if p.tt == FunctionToken {
+			// hoistable declaration
 			exportStmt.Decl = p.parseFuncDecl()
 		} else if p.tt == AsyncToken { // async function or async arrow function
 			async := p.data
 			p.next()
 			if p.tt == FunctionToken && !p.prevLT {
+				// hoistable declaration
 				exportStmt.Decl = p.parseAsyncFuncDecl()
 			} else {
 				// expression
-				exportStmt.Decl = p.parseAsyncExpression(OpExpr, async)
+				exportStmt.Decl = p.parseAsyncExpression(OpAssign, async)
 			}
 		} else if p.tt == ClassToken {
 			exportStmt.Decl = p.parseClassDecl()
@@ -1290,7 +1320,7 @@ func (p *Parser) parseArrayLiteral() (array ArrayExpr) {
 			if spread {
 				p.next()
 			}
-			array.List = append(array.List, Element{p.parseAssignmentExpression(), spread})
+			array.List = append(array.List, Element{p.parseAssignExprOrParam(), spread})
 			prevComma = false
 			if spread && p.tt != CloseBracketToken {
 				p.assumeArrowFunc = false
@@ -1316,7 +1346,7 @@ func (p *Parser) parseObjectLiteral() (object ObjectExpr) {
 		if p.tt == EllipsisToken {
 			p.next()
 			property.Spread = true
-			property.Value = p.parseAssignmentExpression()
+			property.Value = p.parseAssignExprOrParam()
 			if _, isIdent := property.Value.(*Var); !isIdent || p.tt != CloseBraceToken {
 				p.assumeArrowFunc = false
 			}
@@ -1381,7 +1411,7 @@ func (p *Parser) parseObjectLiteral() (object ObjectExpr) {
 				// PropertyName : AssignmentExpression
 				p.next()
 				property.Name = &method.Name
-				property.Value = p.parseAssignmentExpression()
+				property.Value = p.parseAssignExprOrParam()
 			} else if method.Name.IsComputed() || !p.isIdentifierReference(method.Name.Literal.TokenType) {
 				p.fail("object literal", ColonToken, OpenParenToken)
 				return
@@ -1566,14 +1596,14 @@ func (p *Parser) parseAsyncExpression(prec OpPrec, async []byte) IExpr {
 			p.fail("arrow function")
 			return nil
 		} else if p.tt == OpenParenToken {
-			return p.parseParenthesizedExpressionOrArrowFunc(prec, async)
+			return p.parseParenthesizedExpression(prec, async)
 		}
 		left = p.parseAsyncArrowFunc()
 		precLeft = OpAssign
 	} else {
 		left = p.scope.Use(async)
 	}
-	// can be async(), async => ..., or e.g. async + ...
+	// can be async(args), async => ..., or e.g. async + ...
 	return p.parseExpressionSuffix(left, prec, precLeft)
 }
 
@@ -1641,7 +1671,7 @@ func (p *Parser) parseExpression(prec OpPrec) IExpr {
 			}
 			break
 		}
-		suffix := p.parseParenthesizedExpressionOrArrowFunc(prec, nil)
+		suffix := p.parseParenthesizedExpression(prec, nil)
 		p.exprLevel--
 		return suffix
 	case NotToken, BitNotToken, TypeofToken, VoidToken, DeleteToken:
@@ -1972,6 +2002,9 @@ func (p *Parser) parseExpressionSuffix(left IExpr, prec, precLeft OpPrec) IExpr
 		case OptChainToken:
 			if OpCall < prec {
 				return left
+			} else if precLeft < OpCall {
+				p.fail("expression")
+				return nil
 			}
 			p.next()
 			if p.tt == OpenParenToken {
@@ -2123,7 +2156,7 @@ func (p *Parser) parseExpressionSuffix(left IExpr, prec, precLeft OpPrec) IExpr
 			// handle identifier => ..., where identifier could also be yield or await
 			if OpAssign < prec {
 				return left
-			} else if precLeft < OpPrimary {
+			} else if precLeft < OpPrimary || p.prevLT {
 				p.fail("expression")
 				return nil
 			}
@@ -2142,7 +2175,7 @@ func (p *Parser) parseExpressionSuffix(left IExpr, prec, precLeft OpPrec) IExpr
 	}
 }
 
-func (p *Parser) parseAssignmentExpression() IExpr {
+func (p *Parser) parseAssignExprOrParam() IExpr {
 	// this could be a BindingElement or an AssignmentExpression. Here we handle BindingIdentifier with a possible Initializer, BindingPattern will be handled by parseArrayLiteral or parseObjectLiteral
 	if p.assumeArrowFunc && p.isIdentifierReference(p.tt) {
 		tt := p.tt
@@ -2170,53 +2203,40 @@ func (p *Parser) parseAssignmentExpression() IExpr {
 	return p.parseExpression(OpAssign)
 }
 
-func (p *Parser) parseParenthesizedExpressionOrArrowFunc(prec OpPrec, async []byte) IExpr {
+func (p *Parser) parseParenthesizedExpression(prec OpPrec, async []byte) IExpr {
+	// parse ArrowFunc, AsyncArrowFunc, AsyncCallExpr, ParenthesizedExpr
 	var left IExpr
 	precLeft := OpPrimary
 
 	// expect to be at (
 	p.next()
 
-	isAsync := async != nil
+	isAsync := async != nil // prevLT is false before open parenthesis
 	arrowFunc := &ArrowFunc{}
 	parent := p.enterScope(&arrowFunc.Body.Scope, true)
 	prevAssumeArrowFunc, prevIn := p.assumeArrowFunc, p.in
 	p.assumeArrowFunc, p.in = true, true
 
-	// parse a parenthesized expression but assume we might be parsing an (async) arrow function. If this is really an arrow function, parsing as a parenthesized expression cannot fail as AssignmentExpression, ArrayLiteral, and ObjectLiteral are supersets of SingleNameBinding, ArrayBindingPattern, and ObjectBindingPattern respectively. Any identifier that would be a BindingIdentifier in case of an arrow function, will be added as such. If finally this is not an arrow function, we will demote those variables as undeclared and merge them with the parent scope.
+	// parse an Arguments expression but assume we might be parsing an (async) arrow function or ParenthesisedExpression. If this is really an arrow function, parsing as an Arguments expression cannot fail as AssignmentExpression, ArrayLiteral, and ObjectLiteral are supersets of SingleNameBinding, ArrayBindingPattern, and ObjectBindingPattern respectively. Any identifier that would be a BindingIdentifier in case of an arrow function, will be added as such to the scope. If finally this is not an arrow function, we will demote those variables as undeclared and merge them with the parent scope.
 
-	var list []IExpr
-	var rest IExpr
+	rests := 0
+	var args Args
 	for p.tt != CloseParenToken && p.tt != ErrorToken {
-		if p.tt == EllipsisToken && p.assumeArrowFunc {
-			p.next()
-			if isAsync {
-				rest = p.parseAssignmentExpression()
-				if p.tt == CommaToken {
-					p.next()
-				}
-			} else if p.isIdentifierReference(p.tt) {
-				var ok bool
-				rest, ok = p.scope.Declare(ArgumentDecl, p.data)
-				if !ok {
-					p.failMessage("identifier %s has already been declared", string(p.data))
-					return nil
-				}
-				p.next()
-			} else if p.tt == OpenBracketToken {
-				array := p.parseArrayLiteral()
-				rest = &array
-			} else if p.tt == OpenBraceToken {
-				object := p.parseObjectLiteral()
-				rest = &object
-			} else {
-				p.fail("expression")
-				return nil
+		if 0 < len(args.List) && args.List[len(args.List)-1].Rest {
+			// only last parameter can have ellipsis
+			p.assumeArrowFunc = false
+			if !isAsync {
+				p.fail("arrow function", CloseParenToken)
 			}
-			break
 		}
 
-		list = append(list, p.parseAssignmentExpression())
+		rest := p.tt == EllipsisToken
+		if rest {
+			p.next()
+			rests++
+		}
+
+		args.List = append(args.List, Arg{p.parseAssignExprOrParam(), rest})
 		if p.tt != CommaToken {
 			break
 		}
@@ -2227,7 +2247,8 @@ func (p *Parser) parseParenthesizedExpressionOrArrowFunc(prec OpPrec, async []by
 		return nil
 	}
 	p.next()
-	isArrowFunc := p.tt == ArrowToken && p.assumeArrowFunc
+	isArrowFunc := !p.prevLT && p.tt == ArrowToken && p.assumeArrowFunc
+	hasLastRest := 0 < rests && p.assumeArrowFunc
 	p.assumeArrowFunc, p.in = prevAssumeArrowFunc, prevIn
 
 	if isArrowFunc {
@@ -2235,12 +2256,15 @@ func (p *Parser) parseParenthesizedExpressionOrArrowFunc(prec OpPrec, async []by
 		p.await, p.yield = isAsync, false
 
 		// arrow function
-		arrowFunc.Params = Params{List: make([]BindingElement, len(list))}
-		for i, item := range list {
-			arrowFunc.Params.List[i] = p.exprToBindingElement(item) // can not fail when assumArrowFunc is set
-		}
 		arrowFunc.Async = isAsync
-		arrowFunc.Params.Rest = p.exprToBinding(rest)
+		arrowFunc.Params = Params{List: make([]BindingElement, 0, len(args.List)-rests)}
+		for _, arg := range args.List {
+			if arg.Rest {
+				arrowFunc.Params.Rest = p.exprToBinding(arg.Value)
+			} else {
+				arrowFunc.Params.List = append(arrowFunc.Params.List, p.exprToBindingElement(arg.Value)) // can not fail when assumArrowFunc is set
+			}
+		}
 		arrowFunc.Body.List = p.parseArrowFuncBody()
 
 		p.await, p.yield = prevAwait, prevYield
@@ -2248,39 +2272,33 @@ func (p *Parser) parseParenthesizedExpressionOrArrowFunc(prec OpPrec, async []by
 
 		left = arrowFunc
 		precLeft = OpAssign
-	} else if len(list) == 0 && isAsync && rest == nil && prec <= OpCall {
-		// MemberExpression(async) CallExpression()
-		left = p.scope.Use(async)
-		left = &CallExpr{left, Args{}, false}
-		precLeft = OpCall
-	} else if len(list) == 0 && rest == nil || !isAsync && rest != nil || isAsync && OpCall < prec {
+	} else if !isAsync && (len(args.List) == 0 || hasLastRest) {
 		p.fail("arrow function", ArrowToken)
 		return nil
+	} else if isAsync && OpCall < prec || !isAsync && 0 < rests {
+		p.fail("expression")
+		return nil
 	} else {
-		p.exitScope(parent)
-
 		// for any nested FuncExpr/ArrowFunc scope, Parent will point to the temporary scope created in case this was an arrow function instead of a parenthesized expression. This is not a problem as Parent is only used for defining new variables, and we already parsed all the nested scopes so that Parent (not Func) are not relevant anymore. Anyways, the Parent will just point to an empty scope, whose Parent/Func will point to valid scopes. This should not be a big deal.
 		// Here we move all declared ArgumentDecls (in case of an arrow function) to its parent scope as undeclared variables (identifiers used in a parenthesized expression).
+		p.exitScope(parent)
 		arrowFunc.Body.Scope.UndeclareScope()
 
 		if isAsync {
 			// call expression
-			args := Args{}
-			for _, item := range list {
-				args.List = append(args.List, Arg{Value: item, Rest: false})
-			}
-			if rest != nil {
-				args.List = append(args.List, Arg{Value: rest, Rest: true})
-			}
 			left = p.scope.Use(async)
 			left = &CallExpr{left, args, false}
 			precLeft = OpCall
 		} else {
 			// parenthesized expression
-			if 1 < len(list) {
-				left = &GroupExpr{&CommaExpr{list}}
+			if 1 < len(args.List) {
+				commaExpr := &CommaExpr{}
+				for _, arg := range args.List {
+					commaExpr.List = append(commaExpr.List, arg.Value)
+				}
+				left = &GroupExpr{commaExpr}
 			} else {
-				left = &GroupExpr{list[0]}
+				left = &GroupExpr{args.List[0].Value}
 			}
 		}
 	}
