@@ -24,7 +24,8 @@ func TestParse(t *testing.T) {
 		{"#!/usr/bin/env node", "Stmt(#!/usr/bin/env node)"},
 		{"/* comment */", ""},
 		{"/*! comment */", "Stmt(/*! comment */)"},
-		{"5 + /*! comment */ 6", "Stmt(5+6)"},
+		{"5 + /*! comment */ 6", "Stmt(/*! comment */) Stmt(5+6)"},
+		{"var a = /*! comment */ 6", "Stmt(/*! comment */) Decl(var Binding(a = 6))"},
 		{"{}", "Stmt({ })"},
 		{`"use strict"`, `Stmt("use strict")`},
 		{"var a = b;", "Decl(var Binding(a = b))"},
@@ -69,7 +70,10 @@ func TestParse(t *testing.T) {
 		{"for (c && (a in b);;) {}", "Stmt(for (c&&((a in b))) ; ; Stmt({ }))"},
 		{"for (a in b) {}", "Stmt(for a in b Stmt({ }))"},
 		{"for (a = b;;) {}", "Stmt(for (a=b) ; ; Stmt({ }))"},
+		{"for (!a;;) {}", "Stmt(for (!a) ; ; Stmt({ }))"},
 		{"for (var [a] in b) {}", "Stmt(for Decl(var Binding([ Binding(a) ])) in b Stmt({ }))"},
+		{"for await (async of b);", "Stmt(for await async of b Stmt({ }))"},
+		{"for (async of => 5;;);", "Stmt(for (async Params(Binding(of)) => Stmt({ Stmt(return 5) })) ; ; Stmt({ }))"},
 		{"throw 5", "Stmt(throw 5)"},
 		{"try {} catch {b}", "Stmt(try Stmt({ }) catch Stmt({ Stmt(b) }))"},
 		{"try {} finally {c}", "Stmt(try Stmt({ }) finally Stmt({ Stmt(c) }))"},
@@ -188,6 +192,7 @@ func TestParse(t *testing.T) {
 		{"async function a(){ (await a) }", "Decl(async function a Params() Stmt({ Stmt((await a)) }))"},
 		{"x = {async a(b){}}", "Stmt(x={Method(async a Params(Binding(b)) Stmt({ }))})"},
 		{"async(...a)", "Stmt(async(...a))"},
+		{"async(...a,...b)", "Stmt(async(...a, ...b))"},
 
 		// bindings
 		{"let [] = z", "Decl(let Binding([ ] = z))"},
@@ -438,6 +443,7 @@ func TestParseError(t *testing.T) {
 		js  string
 		err string
 	}{
+		{"5a", "unexpected identifier after number"},
 		{"{a", "unexpected EOF"},
 		{"if", "expected ( instead of EOF in if statement"},
 		{"if(a", "expected ) instead of EOF in if statement"},
@@ -458,6 +464,7 @@ func TestParseError(t *testing.T) {
 		{"for(var [a],b;", "unexpected ; in for statement"},
 		{"for(var [a]=5,{b};", "expected = instead of ; in var statement"},
 		{`for(let{[(`, `unexpected EOF in expression`},
+		{"for(async of b);", "expected => instead of b in arrow function"},
 		{"for await", "expected ( instead of EOF in for statement"},
 		{"function a(){for await", "expected ( instead of await in for statement"},
 		{"async function a(){ for await(a;", "expected of instead of ; in for statement"},
@@ -563,6 +570,7 @@ func TestParseError(t *testing.T) {
 		{"export {} from", "expected String instead of EOF in export statement"},
 		{"export {} from", "expected String instead of EOF in export statement"},
 		{"export async", "expected function instead of EOF in export statement"},
+		{"export default async=>a,b", "unexpected , in expression"},
 		{"throw", "unexpected EOF in expression"},
 		{"throw\n", "unexpected newline in throw statement"},
 		{"#private", "expected in instead of EOF in relational expression"},
@@ -581,7 +589,8 @@ func TestParseError(t *testing.T) {
 		{"function a(b = yield c){}", "unexpected c in function declaration"},
 		{"function*a(){ (yield) => yield }", "unexpected => in expression"},
 		{"function*a(){ (yield=5) => yield }", "unexpected = in expression"},
-		{"function*a(){ (...yield) => yield }", "unexpected yield in expression"},
+		{"function*a(){ (...yield) => yield }", "unexpected => in expression"},
+		{"function*a(){ yield yield\n?.() }", "unexpected ?. in expression"},
 		{"x = await\n=> a++", "unexpected => in expression"},
 		{"x=async (await,", "unexpected , in expression"},
 		{"async function a() { class a extends await", "unexpected await in expression"},
@@ -592,7 +601,7 @@ func TestParseError(t *testing.T) {
 		{"async function a() { x = await =>", "unexpected => in expression"},
 		{"async function a(){ (await) => await }", "unexpected ) in expression"},
 		{"async function a(){ (await=5) => await }", "unexpected = in expression"},
-		{"async function a(){ (...await) => await }", "unexpected await in expression"},
+		{"async function a(){ (...await) => await }", "unexpected ) in expression"},
 		{"async+a b", "unexpected b in expression"},
 		{"(async\nfunction(){})", "unexpected function in expression"},
 		{"a + async b", "unexpected b in expression"},
@@ -642,8 +651,8 @@ func TestParseError(t *testing.T) {
 		{"x = ({5: 5}) =>", "unexpected => in expression"},
 		{"x = ({[4+5]: 5}) =>", "unexpected => in expression"},
 		{"x = (a, a) =>", "unexpected => in expression"},
-		{"x = (a, a, ...a) =>", "unexpected ... in expression"},
-		{"x = (a, ...a) =>", "identifier a has already been declared"},
+		{"x = (a, a, ...a) =>", "unexpected => in expression"},
+		{"x = (a, ...a) =>", "unexpected => in expression"},
 		{"(A,{}%0%{})=>0", "invalid parameters in arrow function"},
 		{"({}``=1)=>0", "invalid parameters in arrow function"},
 
