@@ -4,6 +4,14 @@ import (
 	"io"
 )
 
+func isLHSExpr(i IExpr) bool {
+	switch i.(type) {
+	case *CommaExpr, *CondExpr, *YieldExpr, *ArrowFunc, *BinaryExpr, *UnaryExpr:
+		return false
+	}
+	return true
+}
+
 // AsIdentifierName returns true if a valid identifier name is given.
 func AsIdentifierName(b []byte) bool {
 	if len(b) == 0 || !identifierStartTable[b[0]] {
